# SPDX-License-Identifier: BSD-2-Clause

# Copyright (c) 2025 Phil Thompson <phil@riverbankcomputing.com>


def test_base_implementation_int(module):
    class Derived(module.Base):
        def default_value_int(self):
            return 2 * super().default_value_int()

    klass = Derived()
    value = module.Base.default_value_int(klass)

    assert value == 10


def test_py_reimplementation_class(module):
    class Derived(module.Base):
        def default_value_class(self):
            return Derived(b'Derived')

    value = Derived().get_value_class()

    assert module.ispyowned(value)

    # Note that we don't test the type as it will be Base (not Derived) as we
    # haven't implemented %ConvertToSubClassCode.
    assert value.get_class_name() == b'Derived'


def test_py_reimplementation_int(module):
    class Derived(module.Base):
        def default_value_int(self):
            return 2 * super().default_value_int()

    value = Derived().get_value_int()

    assert value == 20



# SPDX-License-Identifier: BSD-2-Clause

# Copyright (c) 2026 Phil Thompson <phil@riverbankcomputing.com>


import pytest


# This is the first test in the entire suite.
def test_free_threading(module, abi_version):
    import sys

    if hasattr(sys.flags, 'gil') and not sys.flags.gil:
        # The interpreter supports free threading.
        if abi_version >= 14:
            # Check that the GIL hasn't been enabled.
            assert not sys._is_gil_enabled()
        else:
            # Check that the GIL has been enabled.
            assert sys._is_gil_enabled()

def test_abstract_instantiation(module):
    with pytest.raises(TypeError):
        module.AbstractBaseClass()

def test_abstract_py_instantiation(module, virtual_hook):
    class Klass(module.AbstractBaseClass):
        pass

    Klass().get_value()
    with pytest.raises(NotImplementedError):
        virtual_hook.reraise()

def test_concrete_instantiation(module):
    assert module.ConcreteClass().get_value() == 10

def test_concrete_py_instantiation(module):
    class Klass(module.AbstractBaseClass):
        def get_value(self):
            return 20

    assert Klass().get_value() == 20
